/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file    adc.c
  * @brief   This file provides code for the configuration
  *          of the ADC instances.
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2024 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */
#include "math.h"
#include "sine.h"
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "adc.h"

/* USER CODE BEGIN 0 */
//---------------- start of debug mode setting ------
#define OPEN_LOOP_TEST
//#define CLOSE_LOOP_TEST

//---------------- end of debug mode setting --------
//---------------- global variable as storage for state variable
float angle=0;
float ua_0 = 0;
float ub_0 = 0;
float uc_0 = 0;
float h = 0.005;
//--------------- global variable for debugging purpose
float sine;
float cosine;
float uref=0; // controller output reference, for debug purpose
float iref=0; // current reference, for debug purpose
float ia,ib,ic;
float adcResult0;
float iaref,ibref,icref;
extern TIM_HandleTypeDef htim1;
#ifndef Ts
	#define Ts 0.00005f
#endif

#ifndef PI
	#define PI 3.1415926536f
#endif

#ifndef SQRT3_DIV_2
	#define SQRT3_DIV_2  0.86602540378 // 近似值表示 sqrt(3)/2
#endif

/* USER CODE END 0 */

ADC_HandleTypeDef hadc1;

/* ADC1 init function */
void MX_ADC1_Init(void)
{

  /* USER CODE BEGIN ADC1_Init 0 */

  /* USER CODE END ADC1_Init 0 */

  ADC_ChannelConfTypeDef sConfig = {0};
  ADC_InjectionConfTypeDef sConfigInjected = {0};

  /* USER CODE BEGIN ADC1_Init 1 */

  /* USER CODE END ADC1_Init 1 */
  /** Configure the global features of the ADC (Clock, Resolution, Data Alignment and number of conversion)
  */
  hadc1.Instance = ADC1;
  hadc1.Init.ClockPrescaler = ADC_CLOCK_SYNC_PCLK_DIV4;
  hadc1.Init.Resolution = ADC_RESOLUTION_12B;
  hadc1.Init.ScanConvMode = ENABLE;
  hadc1.Init.ContinuousConvMode = DISABLE;
  hadc1.Init.DiscontinuousConvMode = DISABLE;
  hadc1.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
  hadc1.Init.ExternalTrigConv = ADC_SOFTWARE_START;
  hadc1.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc1.Init.NbrOfConversion = 1;
  hadc1.Init.DMAContinuousRequests = DISABLE;
  hadc1.Init.EOCSelection = ADC_EOC_SEQ_CONV;
  if (HAL_ADC_Init(&hadc1) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure for the selected ADC regular channel its corresponding rank in the sequencer and its sample time.
  */
  sConfig.Channel = ADC_CHANNEL_0;
  sConfig.Rank = 1;
  sConfig.SamplingTime = ADC_SAMPLETIME_3CYCLES;
  if (HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configures for the selected ADC injected channel its corresponding rank in the sequencer and its sample time
  */
  sConfigInjected.InjectedChannel = ADC_CHANNEL_0;
  sConfigInjected.InjectedRank = 1;
  sConfigInjected.InjectedNbrOfConversion = 2;
  sConfigInjected.InjectedSamplingTime = ADC_SAMPLETIME_3CYCLES;
  sConfigInjected.ExternalTrigInjecConvEdge = ADC_EXTERNALTRIGINJECCONVEDGE_RISING;
  sConfigInjected.ExternalTrigInjecConv = ADC_EXTERNALTRIGINJECCONV_T1_TRGO;
  sConfigInjected.AutoInjectedConv = DISABLE;
  sConfigInjected.InjectedDiscontinuousConvMode = DISABLE;
  sConfigInjected.InjectedOffset = 0;
  if (HAL_ADCEx_InjectedConfigChannel(&hadc1, &sConfigInjected) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configures for the selected ADC injected channel its corresponding rank in the sequencer and its sample time
  */
  sConfigInjected.InjectedChannel = ADC_CHANNEL_1;
  sConfigInjected.InjectedRank = 2;
  if (HAL_ADCEx_InjectedConfigChannel(&hadc1, &sConfigInjected) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN ADC1_Init 2 */

  /* USER CODE END ADC1_Init 2 */

}

void HAL_ADC_MspInit(ADC_HandleTypeDef* adcHandle)
{

  GPIO_InitTypeDef GPIO_InitStruct = {0};
  if(adcHandle->Instance==ADC1)
  {
  /* USER CODE BEGIN ADC1_MspInit 0 */

  /* USER CODE END ADC1_MspInit 0 */
    /* ADC1 clock enable */
    __HAL_RCC_ADC1_CLK_ENABLE();

    __HAL_RCC_GPIOA_CLK_ENABLE();
    /**ADC1 GPIO Configuration
    PA0-WKUP     ------> ADC1_IN0
    PA1     ------> ADC1_IN1
    */
    GPIO_InitStruct.Pin = IA_Pin|IB_Pin;
    GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

    /* ADC1 interrupt Init */
    HAL_NVIC_SetPriority(ADC_IRQn, 0, 0);
    HAL_NVIC_EnableIRQ(ADC_IRQn);
  /* USER CODE BEGIN ADC1_MspInit 1 */

  /* USER CODE END ADC1_MspInit 1 */
  }
}

void HAL_ADC_MspDeInit(ADC_HandleTypeDef* adcHandle)
{

  if(adcHandle->Instance==ADC1)
  {
  /* USER CODE BEGIN ADC1_MspDeInit 0 */

  /* USER CODE END ADC1_MspDeInit 0 */
    /* Peripheral clock disable */
    __HAL_RCC_ADC1_CLK_DISABLE();

    /**ADC1 GPIO Configuration
    PA0-WKUP     ------> ADC1_IN0
    PA1     ------> ADC1_IN1
    */
    HAL_GPIO_DeInit(GPIOA, IA_Pin|IB_Pin);

    /* ADC1 interrupt Deinit */
    HAL_NVIC_DisableIRQ(ADC_IRQn);
  /* USER CODE BEGIN ADC1_MspDeInit 1 */

  /* USER CODE END ADC1_MspDeInit 1 */
  }
}

/* USER CODE BEGIN 1 */
#ifdef OPEN_LOOP_TEST
// Code exclusive to stage 1 test
void HAL_ADCEx_InjectedConvCpltCallback(ADC_HandleTypeDef* hadc)
{
	float ud,uq; // p.u controller output in dq frame
	float ual,ube; // p.u controller output in alpha/beta frame
	float ua,ub,uc;

	float angle_dqframe; //dq 坐标系关于静止alpha/beta坐标系的角度
	float sine_dqframe, cosine_dqframe; //sin(angle_dqframe), cos(angle_dqframe)
	float da,db,dc; //duty cycle for g1,g2,g3
	float freq = 50;
	float u0sv,umax,umin;

	uint32_t adcValue;

    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_11, GPIO_PIN_SET);

	if(hadc == &hadc1)
	{
		/* ---- load local variable from the global one ---*/
		// 将全局变量中的状态变量重新载入本地变量
		angle_dqframe = angle;
		/* ---- end of "load local variable from the global one" ---- */

		/* ---- read the ADC result ---*/
				// 读取本周期采样
		adcValue = HAL_ADCEx_InjectedGetValue(hadc, ADC_INJECTED_RANK_1);
		adcResult0 = adcValue;
		ia = 3.3*(float)adcValue/4095;
		ia -= 1.5;
		ia = ia*2-0.02; //0.071428571428571 = 250*2/1000/7
		ia = (-ia);

		adcValue = HAL_ADCEx_InjectedGetValue(hadc, ADC_INJECTED_RANK_2);
		ib = 3.3*(float)adcValue/4095;
		ib -= 1.5;
		ib = ib*2-0.01;
		ib = -ib;

		//adcValue = HAL_ADCEx_InjectedGetValue(hadc, ADC_INJECTED_RANK_3);
		//ic = (float)adcValue/4095;
		//ic -= 1.5;
		//ic = ic*0.071428571428571+0.074;
		ic = -ia-ib;
		/* ----- end of reading ADC result --------*/



		angle_dqframe += freq*2*PI*Ts;
	/* ---- wrap angle to 0-2pi --- */
        if(angle_dqframe > 2*PI){
			angle_dqframe -= 2*PI;
		}
        if(angle_dqframe < 0){
        	angle_dqframe += 2*PI;
        }
    /* ---- end of wrap angle to 0-2pi --- */

		sine_dqframe = get_sin(angle_dqframe);
		cosine_dqframe = get_cos(angle_dqframe);

		//HAL_DAC_SetValue(&hdac1, DAC_CHANNEL_1, DAC_ALIGN_12B_R, 2048*(m+1));

		ud = uref;
		uq = 0;
		/* invpark transform, 反park变换
		 * ⎡ V_alpha ⎤   ⎡ cos(theta)  -sin(theta) ⎤   ⎡ V_d ⎤
		   ⎢         ⎥ = ⎢                         ⎥ * ⎢     ⎥
		   ⎣ V_beta  ⎦   ⎣ sin(theta)   cos(theta) ⎦   ⎣ V_q ⎦
		*/

		ual = cosine_dqframe*ud - sine_dqframe*uq;
		ube = sine_dqframe*ud + cosine_dqframe*uq;

		/* invclarke transform, 反clarke变换
			[u_a]       [   1      0     1    ]   [u_alpha]
			[u_b]   =   [ -1/2    √3/2   1    ] * [u_beta ]
			[u_c]       [ -1/2   -√3/2   1    ]   [u_0    ]
		*/

		ua = ual;
		ub = -0.5*ual + SQRT3_DIV_2*ube;
		uc = -0.5*ual - SQRT3_DIV_2*ube;

		umin = ua;
		umax = ua;
		if(ub<umin){
			umin = ub;
		}
		if(ub>umax){
			umax = ub;
		}
		if(uc<umin){
			umin = uc;
		}
		if(uc>umax){
			umax = uc;
		}
		/*
		u0sv = -0.5*(umax+umin);
		ua += u0sv;
		ub += u0sv;
		uc += u0sv;
		*/

		// d = (u+1.0)/2;
		da = (0.5+0.5*ua)*999;  // 999 是计数器周期值-1
		db = (0.5+0.5*ub)*999;
		dc = (0.5+0.5*uc)*999;

		// -------- update global variable for monitoring, state updating -----//
		// -------- 更新全局变量由于监控和状态变量更新 ----------//
		angle = angle_dqframe;
		sine = sine_dqframe;
		cosine = cosine_dqframe;

		__HAL_TIM_SetCompare(&htim1, TIM_CHANNEL_1, (uint32_t)da);
		__HAL_TIM_SetCompare(&htim1, TIM_CHANNEL_2, (uint32_t)db);
		__HAL_TIM_SetCompare(&htim1, TIM_CHANNEL_3, (uint32_t)dc);

	    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_11, GPIO_PIN_RESET);

//		__HAL_TIM_SetCompare(&htim1, TIM_CHANNEL_1, 550);
//		__HAL_TIM_SetCompare(&htim1, TIM_CHANNEL_3, 450);

    //HAL_GPIO_TogglePin(GPIOC,GPIO_PIN_13);
	}
}
#endif

#ifdef CLOSE_LOOP_TEST
// Code exclusive to stage 1 test
void HAL_ADCEx_InjectedConvCpltCallback(ADC_HandleTypeDef* hadc)
{
	float idref,iqref,ialref,iberef;
	float ud,uq; // p.u controller output in dq frame
	float ual,ube; // p.u controller output in alpha/beta frame
	float ua,ub,uc;

	float angle_dqframe; //dq 坐标系关于静止alpha/beta坐标系的角度
	float sine_dqframe, cosine_dqframe; //sin(angle_dqframe), cos(angle_dqframe)
	float da,db,dc; //duty cycle for g1,g2,g3
	float freq = 50;
	float u0sv,umax,umin;

	uint32_t adcValue;

    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_11, GPIO_PIN_SET);

	if(hadc == &hadc1)
	{
		/* ---- load local variable from the global one ---*/
		// 将全局变量中的状态变量重新载入本地变量
		angle_dqframe = angle;
		/* ---- end of "load local variable from the global one" ---- */

		/* ---- read the ADC result ---*/
		// 读取本周期采样
		adcValue = HAL_ADCEx_InjectedGetValue(hadc, ADC_INJECTED_RANK_1);
		adcResult0 = adcValue;
		ia = 3.3*(float)adcValue/4095;
		ia -= 1.5;
		ia = ia*2-0.02; //0.071428571428571 = 250*2/1000/7
		ia = (-ia);

		adcValue = HAL_ADCEx_InjectedGetValue(hadc, ADC_INJECTED_RANK_2);
		ib = 3.3*(float)adcValue/4095;
		ib -= 1.5;
		ib = ib*2-0.01;
		ib = -ib;

		//adcValue = HAL_ADCEx_InjectedGetValue(hadc, ADC_INJECTED_RANK_3);
		//ic = (float)adcValue/4095;
		//ic -= 1.5;
		//ic = ic*0.071428571428571+0.074;
		ic = -ia-ib;
		/* ----- end of reading ADC result --------*/

		angle_dqframe += freq*2*PI*Ts;
	/* ---- wrap angle to 0-2pi --- */
        if(angle_dqframe > 2*PI){
			angle_dqframe -= 2*PI;
		}
        if(angle_dqframe < 0){
        	angle_dqframe += 2*PI;
        }
    /* ---- end of wrap angle to 0-2pi --- */

		sine_dqframe = get_sin(angle_dqframe);
		cosine_dqframe = get_cos(angle_dqframe);

		//HAL_DAC_SetValue(&hdac1, DAC_CHANNEL_1, DAC_ALIGN_12B_R, 2048*(m+1));

		idref = iref; //控制器三相占空比幅值输出为0.4
		iqref = 0;
		/* invpark transform, 反park变换
		 * ⎡ V_alpha ⎤   ⎡ cos(theta)  -sin(theta) ⎤   ⎡ V_d ⎤
		   ⎢         ⎥ = ⎢                         ⎥ * ⎢     ⎥
		   ⎣ V_beta  ⎦   ⎣ sin(theta)   cos(theta) ⎦   ⎣ V_q ⎦
		*/

		ialref = cosine_dqframe*idref - sine_dqframe*iqref;
		iberef = sine_dqframe*idref + cosine_dqframe*iqref;

		/* invclarke transform, 反clarke变换
			[u_a]       [   1      0     1    ]   [u_alpha]
			[u_b]   =   [ -1/2    √3/2   1    ] * [u_beta ]
			[u_c]       [ -1/2   -√3/2   1    ]   [u_0    ]
		*/

		iaref = ialref;
		ibref = -0.5*ialref + SQRT3_DIV_2*iberef;
		icref = -0.5*ialref - SQRT3_DIV_2*iberef;

		ua = ua_0;
		ub = ub_0;
		uc = uc_0;

		if(ia>iaref+h){
			ua = -1;
		}
		if(ia<iaref-h){
			ua = 1;
		}

		if(ib>ibref+h){
			ub = -1;
		}
		if(ib<ibref-h){
			ub = 1;
		}

		if(ic>icref+h){
			uc = -1;
		}
		if(ic<icref-h){
			uc = 1;
		}


		// d = (u+1.0)/2;
		da = (0.5+0.5*ua)*999;  // 999 是计数器周期值-1
		db = (0.5+0.5*ub)*999;
		dc = (0.5+0.5*uc)*999;

		// -------- update global variable for monitoring, state updating -----//
		// -------- 更新全局变量由于监控和状态变量更新 ----------//
		angle = angle_dqframe;
		sine = sine_dqframe;
		cosine = cosine_dqframe;

		ua_0 = ua;
		ub_0 = ub;
		uc_0 = uc;

		__HAL_TIM_SetCompare(&htim1, TIM_CHANNEL_1, (uint32_t)da);
		__HAL_TIM_SetCompare(&htim1, TIM_CHANNEL_2, (uint32_t)db);
		__HAL_TIM_SetCompare(&htim1, TIM_CHANNEL_3, (uint32_t)dc);

	    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_11, GPIO_PIN_RESET);

//		__HAL_TIM_SetCompare(&htim1, TIM_CHANNEL_1, 550);
//		__HAL_TIM_SetCompare(&htim1, TIM_CHANNEL_3, 450);

    //HAL_GPIO_TogglePin(GPIOC,GPIO_PIN_13);
	}
}
#endif

/* USER CODE END 1 */
